@@ -1,16 +1,16 @@
 package Unicode::LineBreak;
 
+sub _loadconst { }
 our @MAPs = ();
-
 sub _loadmap {
     my $idx = shift;
     my $map = shift;
     $MAPs[$idx] = $map;
 }
-
 sub _loadrule { }
+sub _packed_hash { return {@_}; }
 
-# _bsearch IDX, VAL
+# _bsearch IDX, VAL, DEFAULT, HASH
 # Examine binary search on property map table with following structure:
 # [
 #     [start, stop, property_value],
@@ -21,10 +21,13 @@ sub _loadrule { }
 sub _bsearch {
     my $map = $MAPs[shift];
     my $val = shift;
+    my $def = shift;
+    my $res = shift;
 
     my $top = 0;
     my $bot = $#{$map};
     my $cur;
+    my $result;
 
     while ($top <= $bot) {
         $cur = $top + int(($bot - $top) / 2);
@@ -34,10 +37,21 @@ sub _bsearch {
         } elsif ($v->[1] < $val) {
             $top = $cur + 1;
         } else {
-            return $v->[2];
+            $result = $v->[2];
+	    last;
         }
     }
-    return undef;
+    $result = $def unless defined $result;
+    my $r = $res->{$result};
+    $result = $r if defined $r;
+    $result;
+}
+
+sub getlbclass {
+    my $self = shift;
+    my $str = shift;
+    return undef unless defined $str and length $str;
+    &_bsearch(0, ord($str), LB_XX, $self->{_lb_hash});
 }
 
 sub getlbrule {
@@ -48,11 +62,15 @@ sub getlbrule {
 
     my $row;
     my $action;
+    my $result = undef;
     if (defined($row = $Unicode::LineBreak::RULES_MAP->[$b_idx]) and
         defined($action = $row->[$a_idx])) {
-	return $action;
+	$result = $action;
     }
-    undef;
+    $result = DIRECT unless defined $result;
+    my $r = $res->{$result};
+    $result = $r if defined $r;
+    $result;
 }
 
 1;
